"""
Tests for vSQL attributes.

The test are done via the Python DB interface.

To run the tests, :mod:`pytest` is required.
"""

from conftest import *


###
### Tests
###

def test_date_year(config_data):
	check_vsql(config_data, "@(2000-02-29).year == 2000")

def test_datetime_year(config_data):
	check_vsql(config_data, "@(2000-02-29T12:34:56).year == 2000")

def test_date_month(config_data):
	check_vsql(config_data, "@(2000-02-29).month == 2")

def test_datetime_month(config_data):
	check_vsql(config_data, "@(2000-02-29T12:34:56).month == 2")

def test_date_day(config_data):
	check_vsql(config_data, "@(2000-02-29).day == 29")

def test_datetime_day(config_data):
	check_vsql(config_data, "@(2000-02-29T12:34:56).day == 29")

def test_datetime_hour(config_data):
	check_vsql(config_data, "@(2000-02-29T12:34:56).hour == 12")

def test_datetime_minute(config_data):
	check_vsql(config_data, "@(2000-02-29T12:34:56).minute == 34")

def test_datetime_second(config_data):
	check_vsql(config_data, "@(2000-02-29T12:34:56).second == 56")

def test_date_weekday(config_data):
	check_vsql(config_data, "@(2000-02-29).weekday == 1")

def test_datetime_weekday(config_data):
	check_vsql(config_data, "@(2000-02-29T12:34:56).weekday == 1")

def test_date_yearday(config_data):
	check_vsql(config_data, "@(2000-02-29).yearday == 60")

def test_datetime_yearday(config_data):
	check_vsql(config_data, "@(2000-02-29T12:34:56).yearday == 60")

def test_datedelta_days(config_data):
	check_vsql(config_data, "days(12).days == 12")

def test_datetimedelta_days(config_data):
	check_vsql(config_data, "timedelta(12, 34).days == 12")

def test_datetimedelta_seconds(config_data):
	check_vsql(config_data, "timedelta(12, 34).seconds == 34")

def test_datetimedelta_total_days(config_data):
	check_vsql(config_data, "timedelta(12, 34).total_days * 60 * 60 * 24 == 12 * 60 * 60 * 24 + 34")

def test_datetimedelta_total_hours(config_data):
	check_vsql(config_data, "timedelta(12, 34).total_hours * 60 * 60 == 12 * 60 * 60 * 24 + 34")

def test_datetimedelta_total_minutes(config_data):
	check_vsql(config_data, "timedelta(12, 34).total_minutes * 60 == 12 * 60 * 60 * 24 + 34")

def test_datetimedelta_total_seconds(config_data):
	check_vsql(config_data, "timedelta(12, 34).total_seconds == 12 * 60 * 60 * 24 + 34")

def test_color_r(config_data):
	check_vsql(config_data, "app.p_color_value.value.r == 0x33")

def test_color_g(config_data):
	check_vsql(config_data, "app.p_color_value.value.g == 0x66")

def test_color_b(config_data):
	check_vsql(config_data, "app.p_color_value.value.b == 0x99")

def test_color_a(config_data):
	check_vsql(config_data, "app.p_color_value.value.a == 0xcc")

def test_geo_lat(config_data):
	check_vsql(config_data, "geo(49.95, 11.59, 'Here').lat == 49.95")

def test_geo_long(config_data):
	check_vsql(config_data, "geo(49.95, 11.59, 'Here').long == 11.59")

def test_geo_info_with_info(config_data):
	check_vsql(config_data, "geo(49.95, 11.59, 'Here').info == 'Here'")

def test_geo_info_without_info(config_data):
	check_vsql(config_data, "geo(49.95, 11.59).info is None")



"""
Tests for the vSQL modulo operator ``%``.

The test are done via the Python DB interface.

To run the tests, :mod:`pytest` is required.
"""

from conftest import *


###
### Tests
###

def test_bool_bool1(config_data):
	check_vsql(config_data, "app.p_bool_none.value % True is None")

def test_bool_bool2(config_data):
	check_vsql(config_data, "app.p_bool_false.value % True == 0")

def test_bool_bool3(config_data):
	check_vsql(config_data, "app.p_bool_true.value % True == 0")

def test_bool_int(config_data):
	check_vsql(config_data, "app.p_bool_true.value % 1 == 0")

def test_bool_number(config_data):
	check_vsql(config_data, "app.p_bool_true.value % 0.3 == 0.1")

def test_int_bool(config_data):
	check_vsql(config_data, "2 % app.p_bool_true.value == 0")

def test_int_int(config_data):
	check_vsql(config_data, "app.p_int_value.value % 2 == 1")

def test_int_number(config_data):
	check_vsql(config_data, "86 % app.p_number_value.value == 1")

def test_number_bool(config_data):
	check_vsql(config_data, "app.p_number_value.value % app.p_bool_true.value == 0.5")

def test_number_int(config_data):
	check_vsql(config_data, "app.p_number_value.value % 4 == 2.5")

def test_number_number1(config_data):
	check_vsql(config_data, "app.p_number_value.value % 3.5 == 0.5")

def test_number_number2(config_data):
	check_vsql(config_data, "app.p_number_value.value % -3.5 == -3.0")

def test_number_number3(config_data):
	check_vsql(config_data, "-app.p_number_value.value % 3.5 == 3.0")

def test_number_number4(config_data):
	check_vsql(config_data, "-app.p_number_value.value % -3.5 == -0.5")

def test_color_color1(config_data):
	check_vsql(config_data, "#000 % app.p_color_value.value == #000")

def test_color_color2(config_data):
	check_vsql(config_data, "app.p_color_value.value % #000 == #28517a")

def test_color_color3(config_data):
	check_vsql(config_data, "#80808080 % rgb(1, 1, 1) == #bfbfbf")



"""
Tests for the vSQL addition operator ``+``.

The test are done via the Python DB interface.

To run the tests, :mod:`pytest` is required.
"""

from conftest import *


###
### Tests
###

d1 = "@(2000-02-29)"
d2 = "@(2000-03-01)"

dt1 = "@(2000-02-29T12:34:56)"
dt2 = "@(2000-03-01T12:34:56)"

def test_bool_bool1(config_data):
	check_vsql(config_data, "app.p_bool_none.value + True is None")

def test_bool_bool2(config_data):
	check_vsql(config_data, "app.p_bool_false.value + True == 1")

def test_bool_bool3(config_data):
	check_vsql(config_data, "app.p_bool_true.value + True == 2")

def test_bool_int(config_data):
	check_vsql(config_data, "app.p_bool_true.value + 1 == 2")

def test_bool_number(config_data):
	check_vsql(config_data, "app.p_bool_true.value + 1.5 == 2.5")

def test_int_bool(config_data):
	check_vsql(config_data, "1 + app.p_bool_true.value == 2")

def test_int_int(config_data):
	check_vsql(config_data, "1 + app.p_int_value.value == 1778")

def test_int_number(config_data):
	check_vsql(config_data, "1 + app.p_number_value.value == 43.5")

def test_str_str1(config_data):
	check_vsql(config_data, "'gurk' + app.p_str_none.value == 'gurk'")

def test_str_str2(config_data):
	check_vsql(config_data, "'gurk' + app.p_str_value.value == 'gurkgurk'")

def test_intlist_intlist(config_data):
	check_vsql(config_data, "[1, 2] + [3, 4] == [1, 2, 3, 4]")

def test_intlist_numberlist(config_data):
	check_vsql(config_data, "[1, 2] + [3.5, 4.5] == [1.0, 2.0, 3.5, 4.5]")

def test_numberlist_intlist(config_data):
	check_vsql(config_data, "[1.5, 2.5] + [3, 4] == [1.5, 2.5, 3.0, 4.0]")

def test_numberlist_numberlist(config_data):
	check_vsql(config_data, "[1.5, 2.5] + [3.5, 4.5] == [1.5, 2.5, 3.5, 4.5]")

def test_strlist_strlist(config_data):
	check_vsql(config_data, "['gurk', 'hurz'] + ['hinz', 'kunz'] == ['gurk', 'hurz', 'hinz', 'kunz']")

def test_datelist_datelist(config_data):
	check_vsql(config_data, "[@(2000-02-29), @(2000-03-01)] + [@(2000-03-02), @(2000-03-03)] == [@(2000-02-29), @(2000-03-01), @(2000-03-02), @(2000-03-03)]")

def test_datetimelist_datetimelist(config_data):
	check_vsql(config_data, "[@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)] + [@(2000-03-02T12:34:56), @(2000-03-03T12:34:56)] == [@(2000-02-29T12:34:56), @(2000-03-01T12:34:56), @(2000-03-02T12:34:56), @(2000-03-03T12:34:56)]")

def test_date_datedelta(config_data):
	check_vsql(config_data, "app.p_date_value.value + days(1) == @(2000-03-01)")

def test_date_monthdelta(config_data):
	check_vsql(config_data, "@(2000-01-31) + months(1) == app.p_date_value.value")

def test_datetime_datedelta(config_data):
	check_vsql(config_data, "app.p_datetime_value.value + days(1) == @(2000-03-01T12:34:56)")

def test_datetime_datetimedelta(config_data):
	check_vsql(config_data, "app.p_datetime_value.value + timedelta(1, 1) == @(2000-03-01T12:34:57)")

def test_datetime_monthdelta(config_data):
	check_vsql(config_data, "@(2000-01-31T12:34:56) + months(1) == app.p_datetime_value.value")

def test_monthdelta_date(config_data):
	check_vsql(config_data, "months(1) + @(2000-01-31) == app.p_date_value.value")

def test_monthdelta_datetime(config_data):
	check_vsql(config_data, "months(1) + @(2000-01-31T12:34:56) == app.p_datetime_value.value")

def test_datedelta_datedelta(config_data):
	check_vsql(config_data, "app.p_datedelta_value.value + days(12) == days(24)")

def test_datedelta_datetimedelta(config_data):
	check_vsql(config_data, "app.p_datedelta_value.value + timedelta(1, 1) == timedelta(13, 1)")

def test_datetimedelta_datedelta(config_data):
	check_vsql(config_data, "app.p_datetimedelta_value.value + days(12) == timedelta(13, (12 * 60 + 34) * 60 + 56)")

def test_datetimedelta_datetimedelta(config_data):
	check_vsql(config_data, "app.p_datetimedelta_value.value + timedelta(2, (12 * 60 + 34) * 60 + 56) == timedelta(4, (1 * 60 + 9) * 60 + 52)")

def test_monthdelta_monthdelta(config_data):
	check_vsql(config_data, "app.p_monthdelta_value.value + months(9) == months(12)")

def test_nulllist_nulllist1(config_data):
	check_vsql(config_data, "[] + [] == []")

def test_nulllist_nulllist2(config_data):
	check_vsql(config_data, "[None, None] + [None] == [None, None, None]")

def test_nulllist_intlist1(config_data):
	check_vsql(config_data, "[] + [1, None, 2] == [1, None, 2]")

def test_nulllist_intlist2(config_data):
	check_vsql(config_data, "[None, None] + [1, None, 2] == [None, None, 1, None, 2]")

def test_nulllist_numberlist1(config_data):
	check_vsql(config_data, "[] + [1.1, None, 2.2] == [1.1, None, 2.2]")

def test_nulllist_numberlist2(config_data):
	check_vsql(config_data, "[None, None] + [1.1, None, 2.2] == [None, None, 1.1, None, 2.2]")

def test_nulllist_strlist1(config_data):
	check_vsql(config_data, "[] + ['gurk', None, 'hurz'] == ['gurk', None, 'hurz']")

def test_nulllist_strlist2(config_data):
	check_vsql(config_data, "[None, None] + ['gurk', None, 'hurz'] == [None, None, 'gurk', None, 'hurz']")

def test_nulllist_datelist1(config_data):
	check_vsql(config_data, f"[] + [{d1}, None, {d2}] == [{d1}, None, {d2}]")

def test_nulllist_datelist2(config_data):
	check_vsql(config_data, f"[None, None] + [{d1}, None, {d2}] == [None, None, {d1}, None, {d2}]")

def test_nulllist_datetimelist1(config_data):
	check_vsql(config_data, f"[] + [{dt1}, None, {dt2}] == [{dt1}, None, {dt2}]")

def test_nulllist_datetimelist2(config_data):
	check_vsql(config_data, f"[None, None] + [{dt1}, None, {dt2}] == [None, None, {dt1}, None, {dt2}]")

def test_intlist_nulllist1(config_data):
	check_vsql(config_data, "[1, None, 2] + [] == [1, None, 2]")

def test_intlist_nulllist2(config_data):
	check_vsql(config_data, "[1, None, 2] + [None, None] == [1, None, 2, None, None]")

def test_numberlist_nulllist1(config_data):
	check_vsql(config_data, "[1.1, None, 2.2] + [] == [1.1, None, 2.2]")

def test_numberlist_nulllist2(config_data):
	check_vsql(config_data, "[1.1, None, 2.2] + [None, None] == [1.1, None, 2.2, None, None]")

def test_strlist_nulllist1(config_data):
	check_vsql(config_data, "['gurk', None, 'hurz'] + [] == ['gurk', None, 'hurz']")

def test_strlist_nulllist2(config_data):
	check_vsql(config_data, "['gurk', None, 'hurz'] + [None, None] == ['gurk', None, 'hurz', None, None]")

def test_datelist_nulllist1(config_data):
	check_vsql(config_data, f"[{d1}, None, {d2}] + [] == [{d1}, None, {d2}]")

def test_datelist_nulllist2(config_data):
	check_vsql(config_data, f"[{d1}, None, {d2}] + [None, None] == [{d1}, None, {d2}, None, None]")

def test_datetimelist_nulllist1(config_data):
	check_vsql(config_data, f"[{dt1}, None, {dt2}] + [] == [{dt1}, None, {dt2}]")

def test_datetimelist_nulllist2(config_data):
	check_vsql(config_data, f"[{dt1}, None, {dt2}] + [None, None] == [{dt1}, None, {dt2}, None, None]")



"""
Tests for the vSQL binary operator ``and``.

The test are done via the Python DB interface.

To run the tests, :mod:`pytest` is required.
"""

from conftest import *


###
### Tests
###

d1 = "@(2000-02-29)"
d2 = "@(2000-03-01)"

dt1 = "@(2000-02-29T12:34:56)"
dt2 = "@(2000-03-01T12:34:56)"

def test_null_bool(config_data):
	check_vsql(config_data, "(None and app.p_bool_true.value) is None")

def test_bool_null(config_data):
	check_vsql(config_data, "(app.p_bool_true.value and None) is None")

def test_bool_bool1(config_data):
	check_vsql(config_data, "(app.p_bool_true.value and app.p_bool_false.value) == False")

def test_bool_bool2(config_data):
	check_vsql(config_data, "(app.p_bool_true.value and app.p_bool_true.value) == True")

def test_int_int1(config_data):
	check_vsql(config_data, "(app.p_int_value.value and 0) == 0")

def test_int_int2(config_data):
	check_vsql(config_data, "(app.p_int_value.value and 42) == 42")

def test_number_number1(config_data):
	check_vsql(config_data, "(app.p_number_value.value and 0.0) == 0.0")

def test_number_number2(config_data):
	check_vsql(config_data, "(app.p_number_value.value and 42.5) == 42.5")

def test_str_str1(config_data):
	check_vsql(config_data, "(app.p_str_none.value and 'gurk') == ''")

def test_str_str2(config_data):
	check_vsql(config_data, "(app.p_str_value.value and 'hurz') == 'hurz'")

def test_date_date1(config_data):
	check_vsql(config_data, "(app.p_date_none.value and @(2000-02-20)) is None")

def test_date_date2(config_data):
	check_vsql(config_data, "(app.p_date_value.value and @(2000-02-20)) == @(2000-02-20)")

def test_datetime_datetime1(config_data):
	check_vsql(config_data, "(app.p_datetime_none.value and @(2000-02-20T12:34:56)) is None")

def test_datetime_datetime2(config_data):
	check_vsql(config_data, "(app.p_datetime_value.value and @(2000-02-20T12:34:56)) == @(2000-02-20T12:34:56)")

def test_datedelta_datedelta1(config_data):
	check_vsql(config_data, "(app.p_datedelta_none.value and days(10)) is None")

def test_datedelta_datedelta2(config_data):
	check_vsql(config_data, "(app.p_datedelta_value.value and days(10)) == days(10)")

def test_datetimedelta_datetimedelta1(config_data):
	check_vsql(config_data, "(app.p_datetimedelta_none.value and hours(12)) is None")

def test_datetimedelta_datetimedelta2(config_data):
	check_vsql(config_data, "(app.p_datetimedelta_value.value and hours(12)) == hours(12)")

def test_intlist_intlist1(config_data):
	check_vsql(config_data, "(0*[1] and [4, 5, 6]) == 0*[1]")

def test_intlist_intlist2(config_data):
	check_vsql(config_data, "([1, 2, 3] and [4, 5, 6]) == [4, 5, 6]")

def test_numberlist_numberlist1(config_data):
	check_vsql(config_data, "(0*[1.1] and [4.4, 5.5, 6.6]) == 0*[1.1]")

def test_numberlist_numberlist2(config_data):
	check_vsql(config_data, "([1.1, 2.2, 3.3] and [4.4, 5.5, 6.6]) == [4.4, 5.5, 6.6]")

def test_nulllist_intlist1(config_data):
	check_vsql(config_data, "([] and [4, 5, 6]) == []")

def test_nulllist_intlist2(config_data):
	check_vsql(config_data, "([None] and [4, 5, 6]) == [4, 5, 6]")

def test_nulllist_numberlist1(config_data):
	check_vsql(config_data, "([] and [4.4, 5.5, 6.6]) == []")

def test_nulllist_numberlist2(config_data):
	check_vsql(config_data, "([None] and [4.4, 5.5, 6.6]) == [4.4, 5.5, 6.6]")

def test_nulllist_strlist1(config_data):
	check_vsql(config_data, "([] and ['gurk', 'hurz']) == []")

def test_nulllist_strlist2(config_data):
	check_vsql(config_data, "([None] and ['gurk', 'hurz']) == ['gurk', 'hurz']")

def test_nulllist_datelist1(config_data):
	check_vsql(config_data, f"([] and [{d1}, {d2}]) == []")

def test_nulllist_datelist2(config_data):
	check_vsql(config_data, f"([None] and [{d1}, {d2}]) == [{d1}, {d2}]")

def test_nulllist_datetimelist1(config_data):
	check_vsql(config_data, f"([] and [{dt1}, {dt2}]) == []")

def test_nulllist_datetimelist2(config_data):
	check_vsql(config_data, f"([None] and [{dt1}, {dt2}]) == [{dt1}, {dt2}]")

def test_intlist_nulllist1(config_data):
	check_vsql(config_data, "([1, 2, 3] and []) == []")

def test_intlist_nulllist2(config_data):
	check_vsql(config_data, "([1, 2, 3] and [None]) == [None]")

def test_numberlist_nulllist1(config_data):
	check_vsql(config_data, "([1.1, 2.2, 3.3] and []) == []")

def test_numberlist_nulllist2(config_data):
	check_vsql(config_data, "([1.1, 2.2, 3.3] and [None]) == [None]")

def test_strlist_nulllist1(config_data):
	check_vsql(config_data, "(['gurk', 'hurz'] and []) == []")

def test_strlist_nulllist2(config_data):
	check_vsql(config_data, "(['gurk', 'hurz'] and [None]) == [None]")

def test_datelist_nulllist1(config_data):
	check_vsql(config_data, f"([{d1}, {d2}] and []) == []")

def test_datelist_nulllist2(config_data):
	check_vsql(config_data, f"([{d1}, {d2}] and [None]) == [None]")

def test_datetimelist_nulllist1(config_data):
	check_vsql(config_data, f"([{dt1}, {dt2}] and []) == []")

def test_datetimelist_nulllist2(config_data):
	check_vsql(config_data, f"([{dt1}, {dt2}] and [None]) == [None]")



"""
Tests for the vSQL unary logical "not" operator ``not``.

The test are done via the Python DB interface.

To run the tests, :mod:`pytest` is required.
"""

from conftest import *


###
### Tests
###

def test_bool1(config_data):
	check_vsql(config_data, "repr(not app.p_bool_none.value) == 'True'")

def test_bool2(config_data):
	check_vsql(config_data, "repr(not app.p_bool_false.value) == 'True'")

def test_bool3(config_data):
	check_vsql(config_data, "repr(not app.p_bool_true.value) == 'False'")

def test_int1(config_data):
	check_vsql(config_data, "repr(not app.p_int_none.value) == 'True'")

def test_int2(config_data):
	check_vsql(config_data, "repr(not app.p_int_value.value) == 'False'")

def test_number1(config_data):
	check_vsql(config_data, "repr(not app.p_number_none.value) == 'True'")

def test_number2(config_data):
	check_vsql(config_data, "repr(not app.p_number_value.value) == 'False'")

def test_str1(config_data):
	check_vsql(config_data, "repr(not app.p_str_none.value) == 'True'")

def test_str2(config_data):
	check_vsql(config_data, "repr(not app.p_str_value.value) == 'False'")

def test_date1(config_data):
	check_vsql(config_data, "repr(not app.p_date_none.value) == 'True'")

def test_date2(config_data):
	check_vsql(config_data, "repr(not app.p_date_value.value) == 'False'")

def test_datetime1(config_data):
	check_vsql(config_data, "repr(not app.p_datetime_none.value) == 'True'")

def test_datetime2(config_data):
	check_vsql(config_data, "repr(not app.p_datetime_value.value) == 'False'")

def test_datedelta1(config_data):
	check_vsql(config_data, "repr(not app.p_datedelta_none.value) == 'True'")

def test_datedelta2(config_data):
	check_vsql(config_data, "repr(not app.p_datedelta_value.value) == 'False'")

def test_datetimedelta1(config_data):
	check_vsql(config_data, "repr(not app.p_datetimedelta_none.value) == 'True'")

def test_datetimedelta2(config_data):
	check_vsql(config_data, "repr(not app.p_datetimedelta_value.value) == 'False'")

def test_monthdelta1(config_data):
	check_vsql(config_data, "repr(not app.p_monthdelta_none.value) == 'True'")

def test_monthdelta2(config_data):
	check_vsql(config_data, "repr(not app.p_monthdelta_value.value) == 'False'")

def test_color1(config_data):
	check_vsql(config_data, "repr(not app.p_color_none.value) == 'True'")

def test_color2(config_data):
	check_vsql(config_data, "repr(not app.p_color_value.value) == 'False'")

def test_geo(config_data):
	check_vsql(config_data, "repr(not geo(49, 11, 'Here')) == 'False'")
